@@ -5,6 +5,8 @@
  * This program is open source, licensed under the PostgreSQL license.
  * For license terms, see the LICENSE file.
  *
+ * Copyright (c) 2014-2017, DALIBO
+ * Copyright (c) 2018-2023, The PoWA-team
  *-------------------------------------------------------------------------
  */
 
@@ -72,9 +74,9 @@ PG_FUNCTION_INFO_V1(powa_stat_user_functions);
 PG_FUNCTION_INFO_V1(powa_stat_all_rel);
 
 #if (PG_VERSION_NUM >= 90500)
-void powa_main(Datum main_arg) pg_attribute_noreturn();
+PGDLLEXPORT void powa_main(Datum main_arg) pg_attribute_noreturn();
 #else
-void powa_main(Datum main_arg) __attribute__((noreturn));
+PGDLLEXPORT void powa_main(Datum main_arg) __attribute__((noreturn));
 #endif
 
 static void powa_sighup(SIGNAL_ARGS);
@@ -118,18 +120,16 @@ compute_powa_frequency(void)
 		local_frequency = 3600000;
 
 	/* Initialize time_powa_frequency to do maths with it */
+#if PG_VERSION_NUM >= 160000
+	time_powa_frequency.ticks = local_frequency / 1000 * NS_PER_S;
+#else
 #ifndef WIN32
-#ifdef HAVE_CLOCK_GETTIME
+	INSTR_TIME_SET_ZERO(time_powa_frequency);
 	time_powa_frequency.tv_sec = local_frequency / 1000; /* Seconds */
-	time_powa_frequency.tv_nsec = 0;
-#else		/* !HAVE_CLOCK_GETTIME */
-	time_powa_frequency.tv_sec = local_frequency / 1000; /* Seconds */
-	time_powa_frequency.tv_usec = 0;
-#endif		/* HAVE_CLOCK_GETTIME */
-
-#else		/* WIN32 */
+#else
 	time_powa_frequency.QuadPart = local_frequency / 1000 * GetTimerFrequency();
 #endif		/* WIN32 */
+#endif		/* pg16+ */
 }
 
 static int64
@@ -265,6 +265,12 @@ powa_main(Datum main_arg)
 	int64		us_to_wait; /* Should be uint64 per postgresql's spec, but we
 							   may have negative result, in our tests */
 
+	if (IsBinaryUpgrade)
+	{
+		elog(LOG, "Binary upgrade mode detected, PoWA worker will exit");
+		proc_exit(0);
+	}
+
 	/* check powa_frequency validity, and if powa is enabled */
 	compute_powa_frequency();
 
@@ -384,6 +390,12 @@ powa_main(Datum main_arg)
 			/* Check if a SIGHUP has been received */
 			powa_process_sighup();
 
+			/*
+			 * We might be in this loop for a long time, so we need to call
+			 * CFI() to promptly respond to any pending event
+			 */
+			CHECK_FOR_INTERRUPTS();
+
 			/*
 			 * Compute if there is still some time to wait (we could have been
 			 * woken up by a latch, or snapshot took more than frequency)
